# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from rich.console import Console
konsol = Console(log_path=False)

import os

ust_dizin_ver = lambda _path, n: os.sep.join(_path.split(os.sep)[:-n])

def dosya_ver(dosya_yolu:str, ust_dizin:int):
    return os.path.join(ust_dizin_ver(__file__, ust_dizin), dosya_yolu)


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from .Libs   import konsol
from eFatura import e_fatura
from sys     import argv

def basla():
    print()

    if len(argv) != 2:
        konsol.print("[bold yellow2][!] Lütfen Vergi Numarası veya TC Kimlik Numarası Belirtin..")
        konsol.print("\n[turquoise2]Örn.: [pale_green1]eFatura 11111111111\n")
        return

    if e_fatura(argv[1]):
        konsol.print(f"[green][+] [light_coral]{argv[1]}[/] Numarası E-Fatura Mükellefidir..\n")
    else:
        konsol.print(f"[red][~] [light_coral]{argv[1]}[/] Numarası E-Fatura Mükellefi Değildir..\n")

if __name__ == "__main__":
    basla()


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

import requests, urllib3, ssl

class CustomHttpAdapter(requests.adapters.HTTPAdapter):
    def __init__(self, ssl_context=None, **kwargs):
        self.ssl_context = ssl_context
        super().__init__(**kwargs)

    def init_poolmanager(self, connections, maxsize, block=False):
        self.poolmanager = urllib3.poolmanager.PoolManager(
            num_pools=connections, maxsize=maxsize,
            block=block, ssl_context=self.ssl_context
        )

def legacy_session():
    ctx         = ssl.create_default_context(ssl.Purpose.SERVER_AUTH)
    ctx.options |= 0x4  # OP_LEGACY_SERVER_CONNECT

    session = requests.session()
    session.mount("https://", CustomHttpAdapter(ctx))
    return session


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from ..Libs.Oturum import legacy_session
from shutil        import copyfileobj
from uuid          import uuid4
from os            import remove
from PIL           import Image
from pytesseract   import image_to_string

def e_fatura(vergi_numarasi:str) -> bool:
    """Vergi veya TC Kimlik Numarasından E-Fatura Mükellefiyet Sorgusu"""
    captcha_resmi = f"captcha_{uuid4()}.jpg"

    while True:
        oturum = legacy_session()
        oturum.headers.update({
            "User-Agent" : "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36"
        })

        captcha_istek = oturum.get("https://sorgu.efatura.gov.tr/kullanicilar/img.php", stream=True)
        with open(captcha_resmi, "wb") as dosya:
            copyfileobj(captcha_istek.raw, dosya)

        captcha_metni = image_to_string(Image.open(captcha_resmi)).strip()
        remove(captcha_resmi)

        e_fatura_istek = oturum.post(
            url  = "https://sorgu.efatura.gov.tr/kullanicilar/xliste.php",
            data = {"search_string": f"{vergi_numarasi}", "captcha_code": f"{captcha_metni}", "submit": "Ara"}
        )
        if "Güvenlik kodu hatalı" in e_fatura_istek.text:
            continue

        return "Mükellef kayıtlıdır" in e_fatura_istek.text


# Bu araç @keyiflerolsun tarafından | @KekikAkademi için yazılmıştır.

from gi import require_version
require_version("Gtk", "3.0")

from gi.repository import Gtk, GLib
from eFatura       import e_fatura
from ..Libs        import dosya_ver

class KekikGUI(Gtk.Window):
    def __init__(self):
        super().__init__()
        self.set_titlebar(Gtk.HeaderBar(
            title             = "eFatura",
            subtitle          = "Mükellefiyet Sorgu Aracı",
            show_close_button = True
        ))
        self.set_resizable(False)
        self.set_default_size(300, 200)
        self.set_position(Gtk.WindowPosition.CENTER)
        self.connect("delete-event", self.pencereyi_kapat)
        self.set_icon_from_file(dosya_ver("Assets/logo.png", ust_dizin=2))

        ayarlar = Gtk.Settings.get_default()
        ayarlar.set_property("gtk-application-prefer-dark-theme", True)

        self.pencere = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=10, margin=20)
        self.add(self.pencere)

        # !
        Program(self)
        # !

    def pencereyi_kapat(self, widget, event):
        dialog = Gtk.MessageDialog(
            parent              = self,
            modal               = True,
            destroy_with_parent = True,
            message_type        = Gtk.MessageType.QUESTION,
            buttons             = Gtk.ButtonsType.OK_CANCEL,
            text                = "Program Kapanıyor",
        )
        dialog.format_secondary_text("Bunu yapmak istediğinize emin misiniz?")
        response = dialog.run()
        dialog.destroy()

        if response == Gtk.ResponseType.OK:
            Gtk.main_quit()
        else:
            return True

    def goster(self):
        self.show_all()
        Gtk.main()


class Program():
    def __init__(self, parent:Gtk.Window):
        self.parent = parent

        sorgu_alani = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=10)
        parent.pencere.pack_start(sorgu_alani, False, False, 0)

        self.arama_metni = Gtk.Entry()
        self.arama_metni.set_placeholder_text("Vergi / TC Kimlik Numarası")
        self.arama_metni.connect("activate", self.ara_butonuna_tiklandiginda)
        sorgu_alani.pack_start(self.arama_metni, False, False, 0)

        self.ara_butonu = Gtk.Button.new_with_label("Ara")
        self.ara_butonu.connect("clicked", self.ara_butonuna_tiklandiginda)
        sorgu_alani.pack_start(self.ara_butonu, False, False, 0)
        self.ara_butonu.grab_focus()

        self.cikti_alani = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=10)
        parent.pencere.pack_start(self.cikti_alani, True, True, 0)

    def ara_butonuna_tiklandiginda(self, widget):
        self.ara_butonu.grab_focus()
        arama_sorgusu = self.arama_metni.get_text()
        self.arama_metni.set_text("")

        self.cikti_alani.foreach(Gtk.Widget.destroy)

        bekleme_etiketi = Gtk.Label()
        bekleme_etiketi.set_markup("<span foreground='#EF7F1A' font_desc='12'>Lütfen Bekleyiniz...</span>")
        bekleme_etiketi.set_margin_top(10)
        bekleme_etiketi.set_halign(Gtk.Align.CENTER)
        bekleme_etiketi.set_justify(Gtk.Justification.CENTER)
        bekleme_etiketi.set_line_wrap(True)
        bekleme_etiketi.set_max_width_chars(30)
        self.cikti_alani.pack_start(bekleme_etiketi, False, False, 0)
        self.parent.show_all()

        def arama_tamamlandi():
            sonuc = e_fatura(arama_sorgusu)
            if sonuc:
                bekleme_etiketi.set_markup(f"<span foreground='#17a2b8' font_desc='12'>{arama_sorgusu} Numarası\nE-Fatura Mükellefidir..</span>")
            else:
                bekleme_etiketi.set_markup(f"<span foreground='#dc3545' font_desc='12'>{arama_sorgusu} Numarası\nE-Fatura Mükellefi Değildir..</span>")
            self.parent.show_all()

        GLib.timeout_add(100, arama_tamamlandi)